import { genId } from './id';

function isType(target, typeString) {
  return typeof target === typeString;
}

export function isUndefined(target) {
  return isType(target, 'undefined');
}

export function isDefined(value){
  return typeof value !== 'undefined';
}

export function isString(target) {
  return isType(target, 'string');
}

export function isNumber(target) {
  return isType(target, 'number');
}

export function toInt(str) {
  return parseInt(str, 10);
}

export function px2rem(size) {
  if (!size) return 'unset'
  const _change = num => {
    const isDigit = /^\d+$|^\d*\.\d+$/g;
    if (isDigit.test(num)) {
      num = num + 'px'
    }
    if (num === 'auto') return num
    return (num + '').indexOf('px') > - 1 ? ((parseFloat(num) / 23.4375) + 'rem') : num
  }
  if (size ==='auto') {
    return size
  }
  let results = size.split(' ')
  results = results.map(item => _change(item))
  return results.join(' ')
}

export const isNaN = Number.isNaN || function isNumberNaN(num) {
  // eslint-disable-next-line no-self-compare
  return num !== num;
}


export function deepFreeze(obj) {
  // 取回定义在obj上的属性名
  const propNames = Object.getOwnPropertyNames(obj);

  // 在冻结自身之前冻结属性
  propNames.forEach((name) => {
    const prop = obj[name];

    // 如果prop是个对象，冻结它
    if (typeof prop === 'object' && prop !== null) deepFreeze(prop);
  });

  // 冻结自身(no-op if already frozen)
  return Object.freeze(obj);
}

// 初始化配置数据并返回所有组件
export function getComponentsAndInitToolsConfig(configTools) {
  // 平铺开所有组
  const componentList = configTools.reduce((preVal, curVal) => [
    ...preVal,
    ...curVal.componentList
  ], []);

  // 注册组件结构
  const data = componentList.reduce((preVal, { componentPack }) => {
    // 修改原数据
    // 生成 From组件和View组件 Name
    const needViewName = !componentPack.componentViewName;
    const needFormName = componentPack.Form && !componentPack.componentFormName;

    // 需要生成viewName 或者 formName
    if (needViewName || needFormName) {
      const id = ((componentPack.propsSchema && (componentPack.propsSchema.id || componentPack.propsSchema.$id)) || genId());
      if (needViewName) componentPack.componentViewName = `View${id}`;
      if (needFormName) componentPack.componentFormName = `Form${id}`;
    }

    if (componentPack.componentFormName) {
      preVal[componentPack.componentFormName] = componentPack.Form;
    }
    preVal[componentPack.componentViewName] = componentPack.View;

    Object.freeze(componentPack);
    return preVal;
  }, {});


  // 冻结配置数据
  Object.freeze(configTools);
  return data;
}
